/**
 * An interface for RQL parsed into nested object form
 *
 * @export
 * @interface RQLOperator
 */
export interface RQLOperator {
    name: string;
    args: Array<RQLOperator | any>;
}
/**
 * User-Defined Type Guard for the RQLOperator interface
 *
 * @export
 * @param arg an unknown value
 * @returns the arg cast as RQLOperator
 */
export declare function isRQLOperator(arg: any): arg is RQLOperator;
export declare const operatorMap: {
    '!=': string;
    '<': string;
    '<=': string;
    '=': string;
    '==': string;
    '>': string;
    '>=': string;
};
/**
 * The main function for parsing an RQL string into object form with typed values
 *
 * @export
 * @param query the RQL string
 * @returns the parsed RQL object
 * @throws {RQLParseError} if the RQL string is invalid for any reason
 * @throws {RQLConversionError} if a string argument can't be converted into a value
 */
export declare function parse(query: string): RQLOperator;
/**
 * Steps through the RQL string character by character identifying operators
 * and their arguments to build RQLOperator objects.
 *
 * @export
 * @param query the RQL string
 * @param [index=0] the current index where we are in the string
 * @param [nestedTopOperator] the parent AND or OR operator under which we are parsing
 * @returns the parsed RQL object
 */
export declare function walkQuery(query: string, index?: number, nestedTopOperator?: RQLOperator): RQLOperator;
/**
 * Helper function for walkQuery -- recursively parse the arguments of a given
 * operator: arguments may be another RQL operator, a value, or an array of values
 *
 * @export
 * @param arg the RQL string argument of an RQL operator
 * @returns the parsed argument: either an RQL operator, a parsed value of
 *  whichever type, or an array of values
 */
export declare function parseArg(arg: string | any[]): RQLOperator | any | any[];
/**
 * Checks wehether argument to an RQL operator may itself be an RQL operator
 *
 * @export
 * @param str the RQL string argument of an RQL operator
 * @returns whether or not this is an RQL operator itself
 */
export declare function isRQLQuery(str: string): boolean;
/**
 * Returns the string of arguments inside of an RQL operator.
 *
 * @export
 * @param str the current portion of an RQL string being parsed, beginning with
 *  the arguments inside of an RQL operator
 * @param [delimiter='\\'] an escape character
 * @returns the arguments of an RQL operator as a string
 * @throws {RQLParseError} if a closing paren is missing
 */
export declare function inside(str: string, delimiter?: string): string;
/**
 * Takes a string of all the arguments of an RQL operator and returns an array of
 * strings of the arguments.
 *
 * @export
 * @param str the string of arguments inside an RQL operator
 * @param [delimiter='\\'] an escape character
 * @returns the arguments as an array of strings or string arrays
 */
export declare function splitArguments(str: string, delimiter?: string): Array<string | any[]>;
/**
 * Removes spaces and matching quotes ('' or "") from around a string
 *
 * @export
 * @param str a string value with or without surrounding spaces or quotes
 * @returns a trimmed string
 */
export declare function trim(str: string): string;
/**
 * Converts a raw RQL string into a standardized format
 *    - changes infix operators to prefix
 *    - removes spaces from around infix operators
 *
 * @export
 * @param query a raw RQL string
 * @returns a normalized RQL string
 * @throws {RQLParseError} in case there is an illegal operator
 */
export declare function normalizeSyntax(query: string): string;
/**
 * Parses an RQL string argument into its typed value using a converter
 *
 * @export
 * @param str an argument of an RQL operator in string format
 * @returns the typed value converted from the string
 * @throws {RQLParseError} if an unknown converter is used
 * @throws {RQLConversionError} if the string can't be converted using the converter
 */
export declare function stringToValue(str: string): any;
