import { RQLOperator } from './parser';
/**
 * A class representing an RQL query.
 *
 * A query consists of an RQL operator which has a name and arguments, where
 * the arguments may themselves be an RQL operator.
 *
 * @export
 * @class RQLQuery
 */
export declare class RQLQuery {
    name: string;
    args: any[];
    /**
     * User-Defined Type Guard for the RQLQuery class
     *
     * @static
     * @param item a variable of an unknown type
     * @returns the value cast as an RQLQuery
     */
    static isRQLQuery(item: unknown): item is RQLQuery;
    /**
     * URL encodes an RQL string
     *
     * @static
     * @param str a string
     * @returns the URL encoded string
     */
    static encodeString(str: string): string;
    /**
     * Encodes any value in an RQL string format
     *
     * @param {any} val a value of any type
     * @returns {string} a string representation of an RQL value
     */
    static encodeValue(val: any): string;
    /**
     * Converts a part of an RQL query from object form into string form
     *
     * @param part part of an RQL query (RQLQuery, a value, or an array of values)
     * @returns {string} returns a string representation of the RQLQuery
     */
    static queryToString(part: unknown): string;
    /**
     * Converts an array of RQLQuery objects or rql values into string form
     *
     * @param args takes an array of RQLQuery or values
     * @param delimiter
     * @returns {string} a string representation of the array of RQLQuery or
     *  values, delimited with the delimiter
     */
    static serializeArgs(args: Array<RQLQuery | any>, delimiter: string): string;
    /**
     * Parse a raw RQL string into an RQLQuery
     *
     * @static
     * @param query
     * @returns
     * @throws {RQLParseError} if the RQL string is invalid for any reason
     * @throws {RQLConversionError} if a string argument can't be converted into a value
     */
    static parse(query: string): RQLQuery;
    /**
     * Converts an RQLOperator object into an RQLQuery, and recursively for each
     * of its args.
     *
     * @static
     * @param obj
     * @returns
     */
    static parseObject(obj: RQLOperator): RQLQuery;
    /**
     * Takes an object of any type: if its an RQLOperator, parse it with
     * [[RQLQuery.parseObject]], otherwise return the raw object value.
     *
     * @static
     * @param obj
     * @returns
     */
    static parseArg(obj: any): any;
    constructor(name: string, args: any[]);
    /**
     * Compares on RQLQuery to another and returns whether they are equal
     *
     * @param b
     * @returns
     */
    equals(b: RQLQuery): boolean;
    /**
     * Converts this RQLQuery into a raw RQL string
     *
     * @returns
     */
    toString(): string;
    /**
     * Converts this RQLQuery into a plain javascript object
     *
     * @returns
     */
    toPlainObject(): RQLOperator;
    /**
     * Converts this RQLQuery into a valid JSON string
     *
     * @returns
     */
    toJSON(): string;
    /**
     * Takes an RQLQuery or a value of any type and adds it to the list of args.
     *
     * @param term
     * @returns
     */
    push(term: RQLQuery | any): this;
    /**
     * This will call the provided function with each RQLQuery in the args of this
     * RQLQuery object, and each of RQLQuery objects in the args of that, and so on.
     * The return value of the provided function is an RQLQuery which will be
     * substituted for the passed in value in the tree.
     *
     * @param {Function} fn a function which takes an RQLQuery and returns a replacement RQLQuery
     * @returns
     */
    walk(fn: (rqlQuery: RQLQuery) => RQLQuery): void;
}
